
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tap.h"
#include "FilmList.h"

int		oldFilmCount;
int		currentFilm;
int		currentWBlock;
dword	sort_x,sort_h;
dword	sortData[MAX_NEW_FILM];
tFilm	sortFilm;

//

void FilmListInit()
{
	memset(oldFilmList, 0xFF, sizeof(oldFilmList));
	memset(newFilmList, 0xFF, sizeof(newFilmList));
	oldFilmCount = 0;
	currentFilm = -1;
	currentWBlock = -1;
}

//
// wenn dieser Film schon in der alten Liste ist, dann in die neue kopieren
//

int FilmListExists( tFile *file )
{
	int i;
	for (i=0; i<oldFilmCount; i++) {
		if (oldFilmList[i].startCluster == file->startCluster &&
			oldFilmList[i].fileDateTime == file->dateTime && currentFilm < MAX_NEW_FILM-1) {
				currentFilm++;
				memcpy(&newFilmList[currentFilm], &oldFilmList[i], sizeof(tFilm));
				return i;
		}
	}
	return -1;
}

tFilm * FilmListGet() {
	return &newFilmList[currentFilm];
}

void FilmListDel() {
	if (currentFilm >= 0) currentFilm--;
}

//
// sucht einen Film in der neuen Liste
//

int FilmListFind( dword startCluster, dword fileDateTime )
{
	int i;
	for (i=0; i<=currentFilm; i++) {
		if (newFilmList[i].startCluster == startCluster &&
			newFilmList[i].fileDateTime == fileDateTime)
				return i;
	}
	return -1;
}

//
// kopiert Filme von alter Liste in die neue Liste
//

void FilmListCopy()
{
	int i;
	for (i=0; i<oldFilmCount; i++) {
		if (FilmListFind(oldFilmList[i].startCluster, oldFilmList[i].fileDateTime) < 0) {
			//Film ist noch nicht in neuer Liste
			if (currentFilm >= MAX_NEW_FILM-1) break;
			currentFilm++;
			memcpy(&newFilmList[currentFilm], &oldFilmList[i], sizeof(tFilm));
		}
	}
}

//
// einen neuen Film hinzufgen
//

int FilmListAddFilm( tFile *file )
{
	if (currentFilm >= MAX_NEW_FILM-1) return -1;
	currentFilm++;
	newFilmList[currentFilm].startCluster = file->startCluster;
	newFilmList[currentFilm].fileDateTime = file->dateTime;
	currentWBlock = -1;
	return currentFilm;
}

//
// einen Werbeblock hinzufgen
//

int FilmListAddBlock( dword start, dword ende )
{
	if (currentWBlock >= MAX_WERBEBLOCK-1) return -1;
	currentWBlock++;
	newFilmList[currentFilm].werbeBlock[currentWBlock].start = start;
	newFilmList[currentFilm].werbeBlock[currentWBlock].ende = ende;
	return currentWBlock;
}

//

void FLSort( int l, int r )
{
	int i,j;
	i=l; j=r;
	sort_x = sortData[(l+r)>>1];
	do {
		while (sortData[i] < sort_x) i++;
		while (sort_x < sortData[j]) j--;
		if (i <= j) {
			sort_h=sortData[i]; sortData[i]=sortData[j]; sortData[j]=sort_h;
			memcpy(&sortFilm, &newFilmList[i], sizeof(tFilm));
			memcpy(&newFilmList[i], &newFilmList[j], sizeof(tFilm));
			memcpy(&newFilmList[j], &sortFilm, sizeof(tFilm));
			i++; j--;
		}
	} while (i <= j);
	if (l < j) FLSort(l,j);
	if (i < r) FLSort(i,r);
}

//
// Neue Filme an den Anfang sortieren
//

void FilmListSort()
{
	int i;
	dword fdt;
	for (i=0; i<MAX_NEW_FILM; i++) {
		fdt = newFilmList[i].fileDateTime;
		if (fdt != 0xFFFFFFFF) fdt = 0xFFFFFFFF-fdt;
		sortData[i] = fdt;
	}
	FLSort(0, MAX_NEW_FILM-1);
}

//
// alte Film-Liste laden
//

int FilmListRead()
{
	int i;
	TYPE_File *listFile;
	FilmListInit();
	ChangeToRoot();
	TAP_Hdd_ChangeDir("ProgramFiles");
	if (!TAP_Hdd_Exist(FILM_LIST_FILE))
		return -1;
	listFile = TAP_Hdd_Fopen(FILM_LIST_FILE);
	if (listFile == NULL) return -2;
	if (listFile->size >= 8800)
		TAP_Hdd_Fread(oldFilmList, sizeof(oldFilmList), 1, listFile);
	TAP_Hdd_Fclose(listFile);
	for (i=0; i<MAX_OLD_FILM; i++)
		if (oldFilmList[i].startCluster != 0xFFFFFFFF) oldFilmCount++;
	return 0;
}

//
// neue Film-Liste speichern
//

int FilmListWrite()
{
	TYPE_File *listFile;
	ChangeToRoot();
	TAP_Hdd_ChangeDir("ProgramFiles");
	if (!TAP_Hdd_Exist(FILM_LIST_FILE))
		TAP_Hdd_Create(FILM_LIST_FILE, ATTR_NORMAL);
	listFile = TAP_Hdd_Fopen(FILM_LIST_FILE);
	if (listFile == NULL) return -1;
	// bei mehr als MAX_OLD_FILM Filmen fallen die ltesten Filme raus
	FilmListCopy();
	FilmListSort();
	TAP_Hdd_Fwrite(newFilmList, sizeof(oldFilmList), 1, listFile);
	TAP_Hdd_Fclose(listFile);
	return 0;
}

