
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tap.h"
#include "Overfly.h"
#include "Pids.h"
#include "LogFile.h"
#include "Letterbox.h"
#include "Drive.h"

#define SLICEMIN	999

extern tPatch		patch;
extern tIniFile		ini;
extern tBookmarks	bookmarks;
extern tAdvertising	werbung;

tLetterbox lbox;

//

void LetterboxInit()
{
	memset(&lbox, 0, sizeof(tLetterbox));
	lbox.size = (dword)(patch.file->size >> 9); //sectors
	if (!ini.bTurbolader) {
		if (ini.bPatchFseek) {
			if (lbox.size > 4088<<11) lbox.size = 4088<<11;
		} else {
			if (lbox.size > 2040<<11) lbox.size = 2040<<11;
		}
	}
	patch.filePos = 8;
	patch.fileSize = lbox.size-(10<<11);
	lbox.step = 192<<11;
	lbox.offset = lbox.step;
	if (lbox.offset > lbox.size) lbox.offset = lbox.size >> 1;
	lbox.minSliceLen = 999;
	lbox.bFindStart = FALSE;
	lbox.bFindEnd = FALSE;
	lbox.bStartPos = FALSE;
	lbox.bAd = FALSE;
}

//

int LetterboxTest()
{
	word sliceLen;

	Hdd_Fseek(lbox.offset);
	sliceLen = PidsFindSlice();
	if (sliceLen > 0 && sliceLen < lbox.minSliceLen) lbox.minSliceLen = sliceLen;
	if (ini.bBlackbox)
		LogFileInt2("LB %d @ %d MB", sliceLen, lbox.offset>>11);
	lbox.offset += lbox.step;
	if (lbox.offset < lbox.size)
		return 2;						// weiter suchen
	if (lbox.minSliceLen < SLICEMIN) {	// SLICEMIN Bytes fr 720*48 Pixel
		patch.letterbox = TRUE;
		return 1;
	} else {
		patch.letterbox = FALSE;
		return 0;
	}
}

void LetterboxSetStart()
{
	dword block = lbox.right / 188;
	werbung.start[werbung.count] = block;
	if (ini.bBlackbox)
		LogFileInt("BM Start %d", block);
	lbox.bStartPos = TRUE;
}

void LetterboxSetEnd()
{
	int len;
	dword block = lbox.left / 188;
	if (!lbox.bStartPos) werbung.start[werbung.count] = 0;
	len = block-werbung.start[werbung.count];
	if (len < 500) return;
	werbung.ende[werbung.count] = block - ini.dejavue;
	if (werbung.count < 9) werbung.count++;
	if (ini.bBlackbox)
		LogFileInt("BM End %d", block);
	lbox.bStartPos = FALSE;
}

//

int LetterboxFinder()
{
	word sliceLen;

	Hdd_Fseek(patch.filePos);
	sliceLen = PidsFindSlice();
	if (sliceLen == 0) {
		lbox.error++;
		if (lbox.error > 10) return 0;
		patch.filePos += 1<<11; // 1MB = 2048 sectors
		if (patch.filePos > patch.fileSize) return 0;
		return 1;
	}
//	if (ini.bBlackbox)
//		LogFileInt2("LB %d @ %d", sliceLen, patch.filePos/188);
	if (lbox.bFindStart) {
		if (lbox.right == 0) {
			if (sliceLen > SLICEMIN) {
				lbox.right = patch.filePos;
				if (ini.bBlackbox)
					LogFileInt2("LB Start [%d .. %d]", lbox.left/188, lbox.right/188);
			} else patch.filePos += 10<<11; // 10MB = 20480 sectors
			return 1;
		}
		if (sliceLen < SLICEMIN)
			lbox.left = patch.filePos;
		else
			lbox.right = patch.filePos;
		patch.filePos = (lbox.left+lbox.right)>>1;
		lbox.iter++;
		if (lbox.iter > 5) {
			LetterboxSetStart();
			lbox.bFindStart = FALSE;
			patch.filePos = lbox.nextPos;
		}
		return 1;
	}
	if (lbox.bFindEnd) {
		if (sliceLen < SLICEMIN)
			lbox.right = patch.filePos;
		else
			lbox.left = patch.filePos;
		patch.filePos = (lbox.left+lbox.right)>>1;
		lbox.iter++;
		if (lbox.iter > 5) {
			LetterboxSetEnd();
			lbox.bFindEnd = FALSE;
			patch.filePos = lbox.nextPos;
		}
		return 1;
	}
	if (sliceLen < SLICEMIN) {
		lbox.loCount++;
		if (lbox.hiCount == 1) lbox.hiCount=0;
		if (lbox.loCount > 7) {
			lbox.hiCount=0;
			if (lbox.bAd) {
				//Test [-80MB .. -70MB]
				lbox.left = patch.filePos - (80<<11);
				lbox.right = patch.filePos - (70<<11);
				lbox.nextPos = patch.filePos;
				patch.filePos = (lbox.left+lbox.right)>>1;
				lbox.iter = 0;
				lbox.bFindEnd = TRUE;
				if (ini.bBlackbox)
					LogFileInt2("LB End [%d .. %d]", lbox.left/188, lbox.right/188);
				lbox.bAd = FALSE;
				return 1;
			}
			patch.filePos += 20<<11;
		} else
			patch.filePos += 10<<11;
	} else {
		lbox.hiCount++;
		if (lbox.hiCount < 2) {
			patch.filePos += 1<<11;
			return 1;
		}
		if (lbox.loCount > 7) {
			lbox.loCount = 0;
			lbox.nextPos = patch.filePos;
			if (patch.filePos > 90<<11)
				patch.filePos -= 90<<11;
			else patch.filePos = 0;
			lbox.left = patch.filePos;
			lbox.right = 0;
			lbox.iter = 0;
			lbox.bFindStart = TRUE;
			lbox.bAd = TRUE;
			return 1;
		}
		lbox.loCount=0;
//		if (lbox.hiCount > 64) return 0;
		if (lbox.hiCount > 256) return 0;
		patch.filePos += 10<<11;
		lbox.bAd = TRUE;
	}
	if (patch.filePos > patch.fileSize) return 0;
	return 1;
}

