
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "tap.h"
#include "Drive.h"
#include "LogFile.h"

#define MAX_PIDS		24
#define MAX_PAKS		58
#define PID_BUFFER		10922
#define BUFFER_OFS		262144
#define FILE_BUFFER		524288
#define FILE_BUFFER2	131072
#define AC3_BUFFER		4096
#define SLICE_BUFFER	4096

#define PID_UNKNOWN		0
#define PID_VIDEO		1
#define PID_AUDIO		2
#define PID_AC3			3
#define PID_VTX			4
#define PID_PCR			5
#define PID_PMT			6
#define PID_PAT			7
#define PID_SDT			8
#define PID_CAT			9

bool	bQuit;
bool	bPmtRead;
bool	bFastMode;
word	vtxPid;
word    pmtPid;
word	pcrPid;
word	videoPid;
word	audioPid;
word	ac3Pid;
word	ac3mode;
word	ac3mode_ext;
word	pidCount;
word	pidList[MAX_PIDS];
word    pidType[MAX_PIDS];
word	pidRate[MAX_PIDS];
word	pidSamp[MAX_PIDS];
word	pidSize[MAX_PIDS];
byte	pidName[MAX_PIDS][8];
byte *  pidBuffer[MAX_PIDS];
byte *  fileBuffer;
byte *  fileBufPos;
byte *  fileBufEnd;
byte *  sliceBuffer;

word	ac3SampRate[]   = { 48000,44100,32000,0 };
word    ac3channels[]   = { 2,1,2,3,3,4,4,5,0 };
word	ac3BitRate[]	= { 32,32,40,40,48,48,56,56,64,64,80,80,96,96,112,112,128,128,160,160,192,
							192,224,224,256,256,320,320,384,384,448,448,512,512,576,576,640,640 };

word	audioSampRate[] = { 44100,48000,32000,0 };
word	audioBitRate[]  = { 0,32,48,56,64,80,96,112,128,160,192,224,256,320,384,0 };



bool PidsSync()
{
	byte c1,c2,c3,c4,c5;
	do {
		c1 = *fileBufPos;
		c2 = *(fileBufPos+188);
		c3 = *(fileBufPos+376);
		c4 = *(fileBufPos+564);
		c5 = *(fileBufPos+752);
		if (c1==0x47 && c2==0x47 && c3==0x47 && c4==0x47 && c5==0x47) return TRUE;
		fileBufPos++;
	} while (fileBufPos < fileBufEnd-800);
	bQuit = TRUE;
	return FALSE;
}

word PidsScanAC3( int idx )
{
	word x,y,typ,br,sr,mode,mode_ext,m,skip;
	dword w,code,len;
	byte *pos;

	w = 0;
	typ = PID_UNKNOWN;
	code = 0xFFFFFFFF;
	pos = pidBuffer[idx];
	len = pidSize[idx];
	do {
		code = (code << 8) | *pos++;
		w++;
	} while (code != 0x01BD && w < len-32);
	if (w >= len-32) {
		code = 0x01BD;
		pos = pidBuffer[idx];
		w = 0;
	}
	if (code == 0x01BD) {
		code = 0;
		do {
			code = ((code << 8) | *pos++) & 0xFFFF;
			w++;
		} while (code != 0x0B77 && w < len-8);
		if (code == 0x0B77) {
			y = (word)*(pos+2); //+4
			br = ac3BitRate[y & 0x3F];
			sr = ac3SampRate[(y >> 6) & 3];
			if (br < 192 || sr < 44100 || w > len-8) return 0;
			pidRate[idx] = br;
			pidSamp[idx] = sr;
			mode = (*(pos+4) >> 5) & 7;
			mode_ext = 0;
			m = (((word)*(pos+4)) & 0xFF) << 8 | *(pos+5);
			skip = 0;
			if ((mode & 1) > 0 && mode != 1) skip++;
			if ((mode & 4) > 0) skip++;
			if (mode == 2) {
				mode_ext |= 6 & m >> (10-(2*skip));  //DS
				skip++;
			}
			mode_ext |= 1 & m >> (12-(2*skip)); //lfe
			ac3mode = mode;
			ac3mode_ext = mode_ext;
			typ = PID_AC3;
		}
	}
	return typ;
}

word PidsScanAudio( int idx )
{
	byte  b;
	word  typ,br,sr;
	dword w,code,cc,len;
	byte *pos;

	w = 0;
	typ = PID_UNKNOWN;
	code = 0xFFFFFFFF;
	pos = pidBuffer[idx];
	len = pidSize[idx];
	do {
		code = (code << 8) | *pos++;
		w++;
	} while ((code < 0x01C0 || code > 0x01CF) && w < len-32);
	if (code >= 0x01C0 && code <= 0x01CF) {
		do {
			do {
				code = (code << 8) | *pos++;
				w++;
				cc = code & 0xFFFF;
			} while ((cc < 0xFFFC || cc > 0xFFFD) && w < len-32);
			cc = code & 0xFFFF;
			if (cc == 0xFFFC || cc == 0xFFFD) {
				//MPG V1 Layer 2
				b = *pos;
				br = audioBitRate[b >> 4];
				sr = audioSampRate[(b >> 2) & 3];
			}
		} while ((br == 0 || sr == 0) && w < len-32);
		if (br > 0 && sr > 0) {
			pidRate[idx] = br;
			pidSamp[idx] = sr;
			typ = PID_AUDIO;
		}
	}
	return typ;
}

void PidsScanBuffer()
{
	word i,typ;
	for (i=0; i<pidCount; i++) {
		if (pidType[i] == PID_UNKNOWN) {
			typ = PidsScanAC3(i);
			if (typ != PID_UNKNOWN) pidType[i] = typ;
			else {
				typ = PidsScanAudio(i);
				if (typ != PID_UNKNOWN) pidType[i] = typ;
			}
		}
		else if (pidType[i] == PID_AUDIO) PidsScanAudio(i);
		else if (pidType[i] == PID_AC3) PidsScanAC3(i);
	}
}

void PidsAddType( word pid, word typ )
{
	word i;
	for (i=0; i<pidCount; i++) {
		if (pidList[i] == pid) {
			pidType[i] = typ;
			return;
		}
	}
	pidList[pidCount] = pid;
	pidType[pidCount] = typ;
	pidCount++;
}

void PidsAddName( word pid, word typ, byte len, byte *addr )
{
	byte k,siz;
	word i;
	siz = len;
	if (siz > 7) siz = 0;
	for (i=0; i<pidCount; i++) {
		if (pidList[i] == pid) {
			for (k=0; k<siz; k++) {
				pidName[i][k] = *addr++;
			}
			pidName[i][siz]=0;
			return;
		}
	}
	pidList[pidCount] = pid;
	pidType[pidCount] = typ;
	for (k=0; k<siz; k++) {
		pidName[pidCount][k] = *addr++;
	}
	pidName[pidCount][siz]=0;
	pidCount++;
}

word PidsReadPMT()
{
	int   pmtlen,infolen,descr_len;
	word  pid,b;
	byte  typ,descr_tag;
	byte  *pos;

	pos = fileBufPos;
	b = (*(pos+6) & 0x0F) << 8;
	pmtlen = b | *(pos+7);
	pos += 15;
	b = (*pos++ & 0x0F) << 8;
	infolen = b | *pos++;
	pos+=infolen;
	pmtlen -= 13;
	if (pmtlen > 160) pmtlen=160;
	do {
		typ = *pos++;
		pid = (*pos<<8 | *(pos+1)) & 0x1FFF;
		pos+=2;
		infolen = (*pos<<8 | *(pos+1)) & 0x0FFF;
		pos+=2;
		pmtlen-=5;
		if (typ == 6 && infolen > 0) {	//AC3(0x6A) oder VTX(0x56)
			do {
				descr_tag = *pos++;
				descr_len = *pos++;
				if (descr_tag == 0x56) {
					vtxPid = pid;	//Videotext Pid
					PidsAddType(pid, PID_VTX);
				}
				if (descr_tag == 0x6A) {
					ac3Pid = pid;	//AC3 Pid
					PidsAddType(pid, PID_AC3);
				}
				if (descr_tag == 0x0A && descr_len >= 2) {
					PidsAddName(pid, PID_AC3, descr_len, pos);
				}
				pos+=descr_len;
				infolen-=(descr_len+2);
			} while (infolen > 0);
		} else if ((typ == 3 || typ == 4) && infolen > 0) {		//MPEG1 Layer2
			do {
				descr_tag = *pos++;
				descr_len = *pos++;
				if (descr_tag == 0x0A && descr_len >= 2) {
					PidsAddType(pid, PID_AUDIO);
					PidsAddName(pid, PID_AUDIO, descr_len, pos);
				}
				pos+=descr_len;
				infolen-=(descr_len+2);
			} while (infolen > 0);
		} else {
			pos+=infolen;
			pmtlen-=infolen;
		}
	} while (pmtlen > 0);
	if (ac3Pid > 0) {
		bPmtRead = TRUE;
		if (bFastMode) bQuit = TRUE;
	}
	return ac3Pid;
}

void PidsCopyPacket( word pid, int idx )
{
	word addfield,addlen;
	byte uc;
	byte *ptr;

	if (pid == videoPid) return;
	if (pidSize[idx] < PID_BUFFER-188) {
		uc = *(fileBufPos+3);
		addfield = ((word)(uc & 0x30))>>4;
		addlen = (addfield>1) ? (((word)*(fileBufPos+4))&0xFF)+1 : 0;
		ptr = pidBuffer[idx]+pidSize[idx];
		if (addlen < 184) {
			pidSize[idx] += (184-addlen);
			memcpy(ptr, fileBufPos+4+addlen, 184-addlen);
		}
	}
}

void PidsAdd( word pid, bool start )
{
	word i;
	if (pidCount < MAX_PIDS) {
		for (i=0; i<pidCount; i++) {
			if (pid == pidList[i]) {
				if (!bFastMode)
					PidsCopyPacket(pid,i);
				if (pidType[i] == PID_PMT && start && !bPmtRead)
					PidsReadPMT();
				return;
			}
		}
		if (pid == videoPid)
			pidType[pidCount] = PID_VIDEO;
		else if (pid == pcrPid)
			pidType[pidCount] = PID_PCR;
		else if (pid == pmtPid && start) {
			pidType[pidCount] = PID_PMT;
			PidsReadPMT();
		}
		else if (pid == 0) {
			pidType[pidCount] = PID_PAT;
		}
		else if (pid == 17)
			pidType[pidCount] = PID_SDT;
		else if (pid == 1)
			pidType[pidCount] = PID_CAT;
		else
			pidType[pidCount] = PID_UNKNOWN;
		pidList[pidCount] = pid;
		pidCount++;
	}
}

word PidsFinder( bool bFast ) {
	bool  start;
	word  i,pid;
	dword count,ofs,len;
	dword heapSize,freeHeapSize,availHeapSize;
	ulong64	size64;
	byte  *pos,*src,*dst;

	TAP_MemInfo(&heapSize, &freeHeapSize, &availHeapSize);
	if (availHeapSize < FILE_BUFFER)
		return 0;
	fileBuffer = TAP_MemAlloc(FILE_BUFFER);
	if (fileBuffer == NULL)
		return 0;
	Hdd_Fseek(0);
	Hdd_Fread(0x0E00, fileBuffer);
	pos = fileBuffer+20;
	pmtPid = (*pos<<8 | *(pos+1)) & 0x1FFF;
	pos+=2;
	pcrPid = (*pos<<8 | *(pos+1)) & 0x1FFF;
	pos+=2;
	videoPid = (*pos<<8 | *(pos+1)) & 0x1FFF;
	pos+=2;
	audioPid = (*pos<<8 | *(pos+1)) & 0x1FFF;
	ac3Pid = vtxPid = ac3mode = ac3mode_ext = 0;
	pidCount = 0;
	bQuit = bPmtRead = FALSE;
	bFastMode = bFast;
	fileBufPos = fileBuffer+BUFFER_OFS;
	fileBufEnd = fileBuffer+FILE_BUFFER;
	Hdd_Fread(FILE_BUFFER-BUFFER_OFS, fileBufPos);
	memset(pidList, 0, 2*MAX_PIDS);
	memset(pidType, 0, 2*MAX_PIDS);
	memset(pidRate, 0, 2*MAX_PIDS);
	memset(pidSamp, 0, 2*MAX_PIDS);
	memset(pidSize, 0, 2*MAX_PIDS);
	memset(pidName, 0, 8*MAX_PIDS);
	for (i=0; i<MAX_PIDS; i++)
		pidBuffer[i] = fileBuffer+(i*PID_BUFFER);
	count=0;
	do {
		PidsSync();
		pid = (((word)*(fileBufPos+1))<<8 | *(fileBufPos+2)) & 0x1FFF;
		start = ((0x40 & *(fileBufPos+1)) != 0) ? TRUE : FALSE;
		PidsAdd(pid,start);
		fileBufPos+=188;
		if (fileBufPos > fileBufEnd-1000) {
			count++;
			dst = fileBuffer+BUFFER_OFS;
			src = fileBufPos;
			len = ((dword)fileBufEnd)-((dword)fileBufPos);
			memcpy(dst, src, len);
			fileBufPos = dst;
			dst += len;
			len = (FILE_BUFFER-BUFFER_OFS)-len;
			Hdd_Fread(len, dst);
		}
	} while (!bQuit && count < 3);
	if (!bFast) PidsScanBuffer();
	if (fileBuffer != NULL) TAP_MemFree(fileBuffer);
	return ac3Pid;
}

word PidsGetAC3Pid()
{
	return ac3Pid;
}

word PidsGetAC3Mode()
{
	return ac3channels[ac3mode]<<4 | (ac3mode_ext & 1);
}

word PidsGetAC3Bitrate()
{
	word i;
	for (i=0; i<pidCount; i++)
		if (pidList[i] == ac3Pid) return pidRate[i];
	return 0;
}

word PidsGetAudioCount()
{
	word i,cnt;
	cnt = 0;
	for (i=0; i<pidCount; i++) {
		if (pidType[i] == PID_AUDIO) cnt++;
	}
	return cnt;
}

word PidsGetAudioPid( int index )
{
	word i,cnt;
	cnt = 0;
	for (i=0; i<pidCount; i++) {
		if (pidType[i] == PID_AUDIO) cnt++;
		if (cnt == index+1) return pidList[i];
	}
	return 0;
}

word PidsGetAudioBitrate( int index )
{
	word i,cnt;
	cnt = 0;
	for (i=0; i<pidCount; i++) {
		if (pidType[i] == PID_AUDIO) cnt++;
		if (cnt == index+1) return pidRate[i];
	}
	return 0;
}

word PidsGetPidSize( word pid )
{
	word i;
	for (i=0; i<pidCount; i++) {
		if (pid == pidList[i]) return pidSize[i];
	}
	return 0;
}

byte * PidsGetPidName( word pid )
{
	word i;
	for (i=0; i<pidCount; i++) {
		if (pid == pidList[i]) return (byte *)&pidName[i][0];
	}
	return NULL;
}

bool PidsIsFreeTV()
{
	word i;
	for (i=0; i<pidCount; i++) {
		if (pidType[i] == PID_CAT) return FALSE;
	}
	return TRUE;
}

//

byte * PidsAllocArea51()
{
	dword heapSize,freeHeapSize,availHeapSize;

	TAP_MemInfo(&heapSize, &freeHeapSize, &availHeapSize);
	if (availHeapSize < FILE_BUFFER2) {
		fileBuffer = NULL;
		return NULL;
	}
	fileBuffer = (byte *) TAP_MemAlloc(FILE_BUFFER2);
	return fileBuffer;
}

void PidsFreeArea51()
{
	if (fileBuffer != NULL) TAP_MemFree(fileBuffer);
	fileBuffer = NULL;
}

word PidsScanArea51( byte *ac3buf )
{
	word w,code,mode,mode_ext,m,skip;

	w = code = mode = mode_ext = 0;
	do {
		code = (code << 8) | *ac3buf++;
		w++;
	} while (code != 0x0B77 && w < AC3_BUFFER);
	if (code == 0x0B77) {
		mode = (*(ac3buf+4) >> 5) & 7;
		m = (((word)*(ac3buf+4)) & 0xFF) << 8 | *(ac3buf+5);
		skip = 0;
		if ((mode & 1) > 0 && mode != 1) skip++;
		if ((mode & 4) > 0) skip++;
		if (mode == 2) {
			mode_ext |= 6 & m >> (10-(2*skip));  //DS
			skip++;
		}
	//	dB = 31 & m >> (7-(2*skip);
		mode_ext |= 1 & m >> (12-(2*skip)); //lfe
	}
	return ac3channels[mode]<<4 | (mode_ext & 1);
}

word PidsFindA51()
{
	word  pid,addfield,addlen,ac3mode;
	byte  b;
	byte *ac3buf;

	bQuit = FALSE;
	ac3buf = fileBuffer;
	fileBufPos = fileBuffer+AC3_BUFFER;
	fileBufEnd = fileBuffer+FILE_BUFFER2-754;
	Hdd_Fread(FILE_BUFFER2-AC3_BUFFER, fileBufPos);
	do {
		PidsSync();
		pid = (((word)*(fileBufPos+1))<<8 | *(fileBufPos+2)) & 0x1FFF;
		if (pid == ac3Pid) {
			b = *(fileBufPos+3);
			addfield = ((word)(b & 0x30))>>4;
			addlen = (addfield>1) ? (((word)*(fileBufPos+4))&0xFF)+1 : 0;
			if (addlen < 184) {
				memcpy(ac3buf, fileBufPos+4+addlen, 184-addlen);
				ac3buf += (184-addlen);
				if (ac3buf > fileBuffer+AC3_BUFFER) bQuit = TRUE;
			}
		}
		fileBufPos+=188;
	} while (!bQuit && fileBufPos < fileBufEnd);
//	LogFileInt("AC3Buf %d", (int)(ac3buf-fileBuffer));
	ac3mode = PidsScanArea51(fileBuffer);
	return ac3mode;
}

word PidsFindArea51()
{
	word ac3mode;
	if (fileBuffer == NULL) return 0;
	ac3mode = PidsFindA51();
	if (ac3mode != 0x20 && ac3mode != 0x51)
		ac3mode = PidsFindA51();
	return ac3mode;
}

//

byte * PidsAllocSlice()
{
	dword heapSize,freeHeapSize,availHeapSize;

	TAP_MemInfo(&heapSize, &freeHeapSize, &availHeapSize);
	if (availHeapSize < FILE_BUFFER) {
		sliceBuffer = NULL;
		return NULL;
	}
	sliceBuffer = (byte *) TAP_MemAlloc(FILE_BUFFER);
	return sliceBuffer;
}

void PidsFreeSlice()
{
	if (sliceBuffer != NULL) TAP_MemFree(sliceBuffer);
	sliceBuffer = NULL;
}

word PidsScanSlice( byte *slcbuf, dword buflen )
{
	word  picType;
	dword vcode;
	byte *ssc;
	byte *bufpos;
	byte *bufend;

	picType = 0;
	vcode = 0xFFFFFFFF;
	ssc = NULL;
	bufpos = slcbuf;
	bufend = slcbuf+buflen;
	do {
		vcode = (vcode << 8) | *bufpos++;
		if (vcode == 0x0100) {
			picType = ((*bufpos <<8 | *(bufpos+1)) & 0x38) >> 3;	//I=1  P=2  B=3
			bufpos+=2;
		}
		if (picType == 1) {
			if (vcode == 0x0102) ssc=bufpos;
			if (vcode == 0x0104) {			// 0102..0104 = 2 Slices
				if (ssc == NULL) return 0;
				return (word)(bufpos-ssc);
			}
		}
	} while (bufpos < bufend);
	return 0;
}

word PidsFindSlc()
{
	dword buflen;
	word  pid,addfield,addlen;
	byte  b;
	byte *slcbuf;

	bQuit = FALSE;
	slcbuf = sliceBuffer;
	fileBufPos = sliceBuffer+SLICE_BUFFER;
	fileBufEnd = sliceBuffer+FILE_BUFFER-754;
	Hdd_Fread(FILE_BUFFER-SLICE_BUFFER, fileBufPos);
	do {
		PidsSync();
		pid = (((word)*(fileBufPos+1))<<8 | *(fileBufPos+2)) & 0x1FFF;
		if (pid == videoPid) {
			b = *(fileBufPos+3);
			addfield = ((word)(b & 0x30))>>4;
			addlen = (addfield>1) ? (((word)*(fileBufPos+4))&0xFF)+1 : 0;
			if (addlen < 184) {
				memcpy(slcbuf, fileBufPos+4+addlen, 184-addlen);
				slcbuf += (184-addlen);
			}
		}
		fileBufPos+=188;
	} while (!bQuit && fileBufPos < fileBufEnd);
	buflen = (dword)(slcbuf-sliceBuffer);
	return PidsScanSlice(sliceBuffer, buflen);
}

word PidsFindSlice()
{
	word len;
	if (sliceBuffer == NULL) return 0;
	len = PidsFindSlc();
	if (len == 0) len = PidsFindSlc();
	return len;
}

